/**
 * This file is loaded via the <script> tag in the index.html file and will
 * be executed in the renderer process for that window. No Node.js APIs are
 * available in this process because `nodeIntegration` is turned off and
 * `contextIsolation` is turned on. Use the contextBridge API in `preload.js`
 * to expose Node.js functionality from the main process.
 */


function HTMLElement(html) {
    let div = document.createElement('div');
    div.innerHTML = html;
    if (div.firstElementChild && div.firstElementChild !== null) {
        return div.firstElementChild;
    }
    else if (div.firstChild && div.firstChild !== null) {
        return div.firstChild;
    }
    let tag = /^<([^\s>]*)?\s?.+?>/gm.exec(html)[1];
    let element = document.createElement(tag);
    let newHtml = html.replace(`<${tag}`, '<div').replace(`</${tag}>`, '</div>');
    let tagProps = HTMLElement(newHtml);
    element.innerHTML = '';
    element.classList.add(...Array.from(tagProps.classList.values()));
    element.style.cssText = document.defaultView.getComputedStyle(tagProps, "").cssText; // broke IE
    for (let attr of tagProps.getAttributeNames()) {
        element.setAttribute(attr, tagProps.getAttribute(attr));
    }
    element.innerText = tagProps.innerText;
    element.innerHTML = tagProps.innerHTML;
    return element;
}

function createElement(type, props) {

    if (props == undefined) {
        props = {};
    }

    let classList = props.class || props.className;
    let text = props.text || props.innerText;
    let html = props.html || props.innerHTML;
    let childs = props.childs || props.childrens || props.elements || [];
    let events = props.events || {};
    let ref = props.ref;

    delete props.class;
    delete props.className;
    delete props.innerText;
    delete props.innerHTML;
    delete props.html;
    delete props.text;
    delete props.childs;
    delete props.childrens;
    delete props.elements;
    delete props.events;
    delete props.ref;

    let elem = HTMLElement(`<${type}${Object.entries(props).map(e => ` ${e[0]}="${e[1]}"`).join(' ')}></${type}>`);

    if (!!classList && !Array.isArray(classList)) {
        classList = classList.split(' ');
    }

    classList && elem.classList.add(...classList);

    for (let eventEntry of Object.entries(events || {})) {
        const [event, eventHandler] = eventEntry;
        elem.addEventListener(event, eventHandler);
    }

    if (text !== undefined) {
        elem.innerText = text;
    }
    else if (html !== undefined) {
        elem.innerHTML = html;
    }

    if (childs !== undefined) {
        if (!Array.isArray(childs)) {
            childs = Object.entries(childs).map((entry) => ({
                $tag: entry[0],
                ...entry[1]
            }));
        }
        for (let child of childs) {
            if (!child.$tag) {
                for (let componentChild of Object.values(child)) {
                    elem.appendChild(componentChild);
                }
            }
            else {
                let tag = child.$tag;
                delete child.$tag;
                if (tag !== undefined) {
                    elem.appendChild(createElement(tag, child));
                }
            }
        }
    }
    if (ref !== undefined) {
        ref(elem);
    }
    return elem;
}

function DialogContent(content, image, events) {

    let root = this;

    function closeModal(e) {
        e.preventDefault();
        e.stopPropagation();
        let modal = e.target.parentElement.parentElement.parentElement;
        modal.style.visibility = 'hidden';
        modal.classList.remove('dye-open');
        modal.innerHTML = '';
        root.clear();
    }

    const buttons = [];

    function handleAction(handler, e) {
        e.preventDefault();
        e.stopPropagation();

        const spinner = document.createElement('div');
        spinner.className = 'spinner';
        spinner.id = 'spinner';

        e.target.innerText = '';
        e.target.disabled = true;
        e.target.style.width = '13%';
        e.target.style.alignItems = 'center';
        e.target.appendChild(spinner);

        const clear = () => {
            spinner.remove();
            e.target.style.width = '23%';
            e.target.innerText = 'Validate';
            e.target.disabled = false;
        }

        let prom = handler(e);

        if (prom && typeof prom.finally == 'function') {
            return prom.finally(clear)
        }

        return clear();
    }

    for (let entry of Object.entries(events)) {
        const [ev, evHandler] = entry;
        buttons.push({
            $tag: 'button',
            class: (ev == 'confirm' ? 'dye-confirm' : 'dye-deny') + ' signup-button',
            text: ev == 'confirm' ? 'Confirm' : 'Deny',
            events: {
                click: handleAction.bind(null, evHandler)
            }
        });
    }

    function render() {
        root.clear();

        const childs = [
            {
                $tag: 'button',
                html: `&times;`,
                class: 'dye-close-btn',
                events: {
                    click: closeModal
                }
            },
            {},
            {
                $tag: 'div',
                class: 'dye-dialog-buttons',
                childs: buttons
            }
        ]

        if (typeof content == 'string') {
            childs[1] = {
                $tag: 'p',
                html: content
            };
        }
        else if (typeof content == 'function') {
            childs[1] = content();
        }
        else if (Array.isArray(content)) {
            childs = [
                childs[0],
                ...content,
                childs[2]
            ]
        }
        else {
            childs[1] = content;
        }

        root.append('div', {
            class: 'dye-modal-content',
            childs: [
                {
                    $tag: 'img',
                    src: `static/${image}`,
                    width: 255
                },
                {
                    $tag: 'div',
                    class: 'dye-speech-bubble',
                    childs: childs
                }
            ]
        });
    }
    render();
    return this.render();
}

function Component(rootFunctions, root) {
    let _component = {};
    let childs = [];
    let onLoadFunc;
    let lastComponent;
    let refs = [];
    let _root = root;
    let _mainroot;
    let _dialogRoot = document.getElementById('dye-modal');

    if (typeof _root !== 'object') {
        _root = document.createElement('div');
    }

    function append(type, props) {
        return childs.push(createElement(type, props));
    }

    function appendComponent(component) {
        if (component instanceof HTMLElement || component.ELEMENT_NODE !== undefined) {
            return childs.push(component);
        }
        else if (component.length == undefined) {
            throw new Error(`invalid component supplied.`);
        }
        return childs.push(...component);
    }

    function cleanRoot() {
        if (typeof root == 'object') {
            for (let child of childs.concat(refs)) {
                if (Array.from(_root.childNodes.values()).some(c => c == child)) {
                    _root.removeChild(child)
                }
            }
            _root.innerHTML = '';
        }
        refs = [];
    }

    function clear() {
        cleanRoot();
        if (_mainroot) {
            for (let child of _mainroot.childNodes) {
                _mainroot.removeChild(child);
            }
        }
        childs = [];
    }

    function onLoad(func) {
        onLoadFunc = func.bind(_component);
    }

    function display(execLoadFunc) {
        if (_root.style.display == 'none') {
            _root.style.display = 'block';
        }
        if (_root.style.visibility == 'hidden') {
            _root.style.visibility = 'unset';
        }
        execLoadFunc && onLoadFunc && onLoadFunc();
    }

    function render(execLoadFunc = true) {

        lastComponent = _root.innerHTML;

        if (_root.style.display == 'block') {
            _root.style.display = 'none';
        }

        cleanRoot();

        for (let child of childs) {
            child && _root.appendChild(child);
        }

        for (let ref of refs) {
            ref && _root.appendChild(ref);
        }

        window.requestAnimationFrame(display.bind(null, execLoadFunc));

        if (root == _root) {
            return _root;
        }

        return _root.childNodes;
    }

    function dialog(options) {
        let {
            image,
            component,
            events
        } = options || {};

        if (image == undefined) {
            image = 'square_9.png';
        }
        if (component == undefined) {
            throw new Error(`invalid component`);
        }

        let root = Component(null, _dialogRoot);
        root.onLoad(() => _dialogRoot.classList.add('dye-open'));

        return DialogContent.call(
            root,
            component,
            image,
            events
        );
    }

    function closeDialog() {
        _dialogRoot.style.visibility = 'hidden';
        _dialogRoot.classList.remove('dye-open');
        _dialogRoot.innerHTML = '';
    }

    function ref() {
        let newRef = createElement("div");
        refs.push(newRef);
        return Component(null, newRef);
    }

    Object.defineProperties(_component, {
        root: { get: () => _root },
        append: { get: () => append },
        appendComponent: { get: () => appendComponent },
        render: { get: () => render },
        dialog: { get: () => dialog },
        closeDialog: { get: () => closeDialog },
        clear: { get: () => clear },
        onLoad: { get: () => onLoad },
        newRef: { get: () => ref },
        dialogRoot: { get: () => Component(null, _dialogRoot) }
    });

    return _component;
}

const loader_body = document.getElementById('body-loader');
const side_panel = document.getElementById('sidePanel');
const rootDoc = document.getElementById('root');
const rootHeaderDoc = document.getElementById('rootHeader');

function loader(display = false) {
    if (display) {
        loader_body.classList.add('loader', 'loading');
    }
    else {
        loader_body.classList.remove('loader', 'loading');
    }
}

function togglePanel() {
    side_panel.classList.toggle('active');
}

function HeaderComponent(title, showAccount = false) {
    this.clear();
    this.append('div', {
        class: 'header-left',
        text: title
    });
    const childs = [];
    if (showAccount) {
        childs.push({
            $tag: 'div',
            class: "account-btn",
            html: `👤 Accounts`,
            events: {
                click: function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                    return togglePanel();
                }
            }
        })
    }
    this.append('div', {
        class: 'header-right',
        childs
    });
    return this.render();
}

function BodyComponent(news, onPlay) {
    this.clear();

    for (let newsItem of news) {
        const { title, date, description } = newsItem;
        this.append('div', {
            class: 'news',
            childs: [
                {
                    $tag: 'img',
                    src: 'assets/1762548_67f45ce0661a5.jpg',
                },
                {
                    $tag: 'div',
                    class: 'news-content',
                    childs: {
                        h3: { text: title },
                        div: { class: 'date', text: date },
                        p: { text: description }
                    }
                }
            ]
        })
    }
    //window.playWindowOpen 
    const playProps = {
        class: 'play-btn',
        html: `▶ Play`,
        events: {
            click: function (e) {
                e.preventDefault();
                e.stopPropagation();
                return onPlay();
            }
        }
    };

    if (window.playWindowOpen) {
        playProps.disabled = 'true';
        playProps.style = 'background: #808080f2';
        playProps.html = 'Playing';
    }

    this.append('div', playProps);
    return this.render();
}

function AccountsComponent(accounts) {
    this.clear();
    this.append('div', {
        class: 'panel-title',
        text: 'Manage Accounts'
    });

    for (let account of accounts) {
        this.append('div', {
            class: 'account-card',
            childs: [
                {
                    $tag: 'div',
                    class: 'account-info',
                    childs: [
                        {
                            $tag: 'div',
                            class: 'account-username',
                            text: `${account.pub.slice(0, 12)}...`
                        }
                    ]
                },
                {
                    $tag: 'div',
                    class: 'account-actions',
                    childs: {
                        button: {
                            class: 'trash-button',
                            html: ` <svg class="trash-icon" viewBox="0 0 24 24">
            <path d="M3 6h18M9 6v12m6-12v12M5 6l1 14a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2l1-14" stroke="currentColor"
              stroke-width="2" fill="none" stroke-linecap="round" stroke-linejoin="round" />
            <path d="M10 11v6M14 11v6" stroke="currentColor" stroke-width="2" fill="none" stroke-linecap="round"
              stroke-linejoin="round" />
          </svg>`,
                            events: {
                                click: function (e) {
                                    e.preventDefault();
                                    e.stopPropagation();
                                    return clientApi.call('delete', account.pub).finally(() => {
                                        loader(false);
                                        window.location.reload();
                                    });
                                }
                            }
                        }
                    }
                }
            ]
        })
    }
    this.append('button', {
        class: 'panel-button',
        text: 'Add Account'
    })
    return this.render();
}

const mainHeaderRoot = Component(null, rootHeaderDoc);
const mainBodyRoot = Component(null, rootDoc);

const changeHeader = (title, showAccount = false) => HeaderComponent.call(mainHeaderRoot, title, showAccount);
const setAccounts = (accounts = []) => AccountsComponent.call(Component(null, side_panel), accounts);
const setBody = (news, onPlay, shouldUpdate = false) => BodyComponent.call(Component(null, rootDoc), news, onPlay);

function deleteAccount(btn) {
    const card = btn.closest('.account-card');
    card.remove();
}

loader(true);
changeHeader('Retro: Loading...');
loader(false);

function onPlay() {
    if (window.playWindowOpen) {
        return;
    }
    loader(true);
    return clientApi.call('play').finally(() => {
        window.playWindowOpen = true;
        loader(false);
    });
}

setBody([], onPlay);